@@ -9,20 +9,20 @@ Scope::Upper - Act on upper scopes.
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
 our $VERSION;
 BEGIN {
- $VERSION = '0.05';
+ $VERSION = '0.06';
 }
 
 =head1 SYNOPSIS
 
     package X;
 
-    use Scope::Upper qw/reap localize localize_elem localize_delete/;
+    use Scope::Upper qw/reap localize localize_elem localize_delete :words/;
 
     sub desc { shift->{desc} }
 
@@ -30,21 +30,22 @@ BEGIN {
      my ($desc) = @_;
 
      # First localize $x so that it gets destroyed last
-     localize '$x' => bless({ desc => $desc }, __PACKAGE__) => 1;
+     localize '$x' => bless({ desc => $desc }, __PACKAGE__) => UP; # one scope up
 
      reap sub {
       my $pkg = caller;
       my $x = do { no strict 'refs'; ${$pkg.'::x'} }; # Get the $x in the scope
       print $x->desc . ": done\n";
-     } => 1;
+     } => SCOPE 1; # same as UP here
 
      localize_elem '%SIG', '__WARN__' => sub {
       my $pkg = caller;
       my $x = do { no strict 'refs'; ${$pkg.'::x'} }; # Get the $x in the scope
       CORE::warn($x->desc . ': ' . join('', @_));
-     } => 1;
+     } => UP CALLER 0; # same as UP here
 
-     localize_delete '@ARGV', $#ARGV => 1; # delete last @ARGV element
+     # delete last @ARGV element
+     localize_delete '@ARGV', -1 => UP SUB HERE; # same as UP here
     }
 
     package Y;
@@ -84,6 +85,15 @@ You can also return to an upper level and know which context was in use then.
 
 =head1 FUNCTIONS
 
+In all those functions, C<$context> refers to the target scope.
+
+You have to use one or a combination of L</WORDS> to build the C<$context> to pass to these functions.
+This is needed in order to ensure that the module still works when your program is ran in the debugger.
+Don't try to use a raw value or things will get messy.
+
+The only thing you can assume is that it is an I<absolute> indicator of the frame.
+This means that you can safely store it at some point and use it when needed, and it will still denote the original scope.
+
 =cut
 
 BEGIN {
@@ -91,13 +101,13 @@ BEGIN {
  XSLoader::load(__PACKAGE__, $VERSION);
 }
 
-=head2 C<reap $callback, $level>
+=head2 C<reap $callback, $context>
 
-Add a destructor that calls C<$callback> when the C<$level>-th upper scope ends, where C<0> corresponds to the current scope.
+Add a destructor that calls C<$callback> when the upper scope represented by C<$context> ends.
 
-=head2 C<localize $what, $value, $level>
+=head2 C<localize $what, $value, $context>
 
-A C<local> delayed to the time of first return into the C<$level>-th upper scope.
+A C<local> delayed to the time of first return into the upper scope denoted by C<$context>.
 C<$what> can be :
 
 =over 4
@@ -114,7 +124,7 @@ A string beginning with a sigil, representing the symbol to localize and to assi
 If the sigil is C<'$'>, L</localize> follows the same syntax as C<local $x = $value>, i.e. C<$value> isn't dereferenced.
 For example,
 
-    localize '$x', \'foo' => 0;
+    localize '$x', \'foo' => HERE;
 
 will set C<$x> to a reference to the string C<'foo'>.
 Other sigils (C<'@'>, C<'%'>, C<'&'> and C<'*'>) require C<$value> to be a reference of the corresponding type.
@@ -122,19 +132,19 @@ Other sigils (C<'@'>, C<'%'>, C<'&'> and C<'*'>) require C<$value> to be a refer
 When the symbol is given by a string, it is resolved when the actual localization takes place and not when C<localize> is called.
 This means that
 
-    sub tag { localize '$x', $_[0] => 1; }
+    sub tag { localize '$x', $_[0] => UP }
 
 will localize in the caller's namespace.
 
 =back
 
-=head2 C<localize_elem $what, $key, $value, $level>
+=head2 C<localize_elem $what, $key, $value, $context>
 
 Similar to L</localize> but for array and hash elements.
 If C<$what> is a glob, the slot to fill is determined from which type of reference C<$value> is ; otherwise it's inferred from the sigil.
 C<$key> is either an array index or a hash key, depending of which kind of variable you localize.
 
-=head2 C<localize_delete $what, $key, $level>
+=head2 C<localize_delete $what, $key, $context>
 
 Similiar to L</localize>, but for deleting variables or array/hash elements.
 C<$what> can be:
@@ -157,77 +167,82 @@ C<$key> is ignored.
 
 =back
 
-=head2 C<unwind @values, $level>
+=head2 C<unwind @values, $context>
 
-Returns C<@values> I<from> the context indicated by C<$level>, i.e. from the subroutine, eval or format just above C<$level>.
+Returns C<@values> I<from> the context pointed by C<$context>, i.e. from the subroutine, eval or format just above C<$context>.
 
-The upper level isn't coerced onto C<@values>, which is hence always evaluated in list context.
+The upper context isn't coerced onto C<@values>, which is hence always evaluated in list context.
 This means that
 
     my $num = sub {
      my @a = ('a' .. 'z');
-     unwind @a => 0;
+     unwind @a => HERE;
     }->();
 
 will set C<$num> to C<'z'>.
 You can use L</want_at> to handle these cases.
 
-=head2 C<want_at $level>
+=head2 C<want_at $context>
 
-Like C<wantarray>, but for the subroutine/eval/format context just above C<$level>.
+Like C<wantarray>, but for the subroutine/eval/format just above C<$context>.
 
 The previous example can then be "corrected" :
 
     my $num = sub {
      my @a = ('a' .. 'z');
-     unwind +(want_at(0) ? @a : scalar @a) => 0;
+     unwind +(want_at(HERE) ? @a : scalar @a) => HERE;
     }->();
 
 will righteously set C<$num> to C<26>.
 
 =head1 WORDS
 
-=head2 C<TOP>
+=head2 Constants
 
-Returns the level that currently represents the highest scope.
+=head3 C<TOP>
 
-=head2 C<HERE>
+Returns the context that currently represents the highest scope.
 
-The current level - i.e. C<0>.
+=head3 C<HERE>
 
-=head2 C<UP $from>
+The context of the current scope.
 
-The level of the scope just above C<$from>.
+=head2 Getting a context from a context
 
-=head2 C<DOWN $from>
+For any of those functions, C<$from> is expected to be a context.
+When omitted, it defaults to the the current context.
 
-The level of the scope just below C<$from>.
+=head3 C<UP $from>
 
-=head2 C<SUB $from>
+The context of the scope just above C<$from>.
 
-The level of the closest subroutine context above C<$from>.
+=head3 C<SUB $from>
 
-=head2 C<EVAL $from>
+The context of the closest subroutine above C<$from>.
 
-The level of the closest eval context above C<$from>.
+=head3 C<EVAL $from>
 
-If C<$from> is omitted in any of those functions, the current level is used as the reference level.
+The context of the closest eval above C<$from>.
 
-=head2 C<CALLER $stack>
+=head2 Getting a context from a level
 
-The level of the C<$stack>-th upper subroutine/eval/format context.
-It kind of corresponds to the context represented by C<caller $stack>, but while e.g. C<caller 0> refers to the caller context, C<CALLER 0> will refer to the top scope in the current context.
-For example,
+Here, C<$level> should denote a number of scopes above the current one.
+When omitted, it defaults to C<0> and those functions return the same context as L</HERE>.
 
-    reap ... => CALLER(0)
+=head3 C<SCOPE $level>
 
-will fire the destructor when the current subroutine/eval/format ends.
+The C<$level>-th upper context, regardless of its type.
+
+=head3 C<CALLER $level>
+
+The context of the C<$level>-th upper subroutine/eval/format.
+It kind of corresponds to the context represented by C<caller $level>, but while e.g. C<caller 0> refers to the caller context, C<CALLER 0> will refer to the top scope in the current context.
 
 =head1 EXPORT
 
 The functions L</reap>, L</localize>, L</localize_elem>, L</localize_delete>,  L</unwind> and L</want_at> are only exported on request, either individually or by the tags C<':funcs'> and C<':all'>.
 
-Same goes for the words L</TOP>, L</HERE>, L</UP>, L</DOWN>, L</SUB>, L</EVAL> and L</CALLER> that are only exported on request, individually or by the tags C<':words'> and C<':all'>.
+Same goes for the words L</TOP>, L</HERE>, L</UP>, L</SUB>, L</EVAL>, L</SCOPE> and L</CALLER> that are only exported on request, individually or by the tags C<':words'> and C<':all'>.
 
 =cut
 
@@ -236,7 +251,7 @@ use base qw/Exporter/;
 our @EXPORT      = ();
 our %EXPORT_TAGS = (
  funcs => [ qw/reap localize localize_elem localize_delete unwind want_at/ ],
- words => [ qw/TOP HERE UP DOWN SUB EVAL CALLER/ ],
+ words => [ qw/TOP HERE UP SUB EVAL SCOPE CALLER/ ],
 );
 our @EXPORT_OK   = map { @$_ } values %EXPORT_TAGS;
 $EXPORT_TAGS{'all'} = [ @EXPORT_OK ];
@@ -248,7 +263,7 @@ Consider those examples:
 
     local $x = 0;
     {
-     reap sub { print $x } => 0;
+     reap sub { print $x } => HERE;
      local $x = 1;
      ...
     }
@@ -256,7 +271,7 @@ Consider those examples:
     ...
     {
      local $x = 1;
-     reap sub { $x = 2 } => 0;
+     reap sub { $x = 2 } => HERE;
      ...
     }
     # $x is 0
@@ -266,6 +281,9 @@ The first case is "solved" by moving the C<local> before the C<reap>, and the se
 L</reap>, L</localize> and L</localize_elem> effects can't cross C<BEGIN> blocks, hence calling those functions in C<import> is deemed to be useless.
 This is an hopeless case because C<BEGIN> blocks are executed once while localizing constructs should do their job at each run.
 
+Some rare oddities may still happen when running inside the debugger.
+It may help to use a perl higher than 5.8.9 or 5.10.0, as they contain some context fixes.
+
 =head1 DEPENDENCIES
 
 L<XSLoader> (standard since perl 5.006).
@@ -296,6 +314,8 @@ Tests code coverage report is available at L<http://www.profvince.com/perl/cover
 
 Inspired by Ricardo Signes.
 
+Thanks to Shawn M. Moore for motivation.
+
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008-2009 Vincent Pit, all rights reserved.
